@@ -1,4 +1,4 @@
-# (C) Copyright 2014-2016 Hewlett Packard Enterprise Development Company LP
+# (C) Copyright 2014-2016 Hewlett Packard Enterprise Development LP
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,8 +33,11 @@ group_by_types = ['alarm_definition_id', 'name', 'state', 'severity',
                   'link', 'lifecycle_state', 'metric_name',
                   'dimension_name', 'dimension_value']
 allowed_notification_sort_by = {'id', 'name', 'type', 'address', 'created_at', 'updated_at'}
-allowed_alarm_sort_by = {'alarm_id', 'alarm_definition_id', 'state', 'severity', 'lifecycle_state', 'link',
-                         'state_updated_timestamp', 'updated_timestamp', 'created_timestamp'}
+allowed_alarm_sort_by = {'alarm_id', 'alarm_definition_id',
+                         'alarm_definition_name', 'state', 'severity',
+                         'lifecycle_state', 'link',
+                         'state_updated_timestamp', 'updated_timestamp',
+                         'created_timestamp'}
 allowed_definition_sort_by = {'id', 'name', 'severity', 'updated_at', 'created_at'}
 
 # Notification valid types
@@ -121,6 +124,7 @@ def do_metric_create_raw(mc, args):
            help="Retrieve data for the specified tenant/project id instead of "
                 "the tenant/project from the user's Keystone credentials.")
 def do_metric_name_list(mc, args):
+    '''List names of metrics.'''
     fields = {}
     if args.dimensions:
         fields['dimensions'] = utils.format_dimensions_query(args.dimensions)
@@ -213,6 +217,87 @@ def do_metric_list(mc, args):
                 formatters=formatters)
 
 
+@utils.arg('--metric-name', metavar='<METRIC_NAME>',
+           help='Name of the metric to report dimension name list.',
+           action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum '
+                'limit.')
+@utils.arg('--tenant-id', metavar='<TENANT_ID>',
+           help="Retrieve data for the specified tenant/project id instead of "
+                "the tenant/project from the user's Keystone credentials.")
+def do_dimension_name_list(mc, args):
+    '''List names of metric dimensions.'''
+    fields = {}
+    if args.metric_name:
+        fields['metric_name'] = args.metric_name
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.tenant_id:
+        fields['tenant_id'] = args.tenant_id
+
+    try:
+        dimension_names = mc.metrics.list_dimension_names(**fields)
+    except exc.HTTPException as he:
+        raise exc.CommandError(
+            'HTTPException code=%s message=%s' %
+            (he.code, he.message))
+
+    if args.json:
+        print(utils.json_formatter(dimension_names))
+        return
+
+    if isinstance(dimension_names, list):
+        utils.print_list(dimension_names, ['Dimension Names'], formatters={
+            'Dimension Names': lambda x: x['dimension_name']})
+
+
+@utils.arg('dimension_name', metavar='<DIMENSION_NAME>',
+           help='Name of the dimension to list dimension values.')
+@utils.arg('--metric-name', metavar='<METRIC_NAME>',
+           help='Name of the metric to report dimension value list.',
+           action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum '
+                'limit.')
+@utils.arg('--tenant-id', metavar='<TENANT_ID>',
+           help="Retrieve data for the specified tenant/project id instead of "
+                "the tenant/project from the user's Keystone credentials.")
+def do_dimension_value_list(mc, args):
+    '''List names of metric dimensions.'''
+    fields = {}
+    fields['dimension_name'] = args.dimension_name
+    if args.metric_name:
+        fields['metric_name'] = args.metric_name
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.tenant_id:
+        fields['tenant_id'] = args.tenant_id
+
+    try:
+        dimension_values = mc.metrics.list_dimension_values(**fields)
+    except exc.HTTPException as he:
+        raise exc.CommandError(
+            'HTTPException code=%s message=%s' %
+            (he.code, he.message))
+
+    if args.json:
+        print(utils.json_formatter(dimension_values))
+        return
+
+    if isinstance(dimension_values, list):
+        utils.print_list(dimension_values, ['Dimension Values'], formatters={
+            'Dimension Values': lambda x: x['dimension_value']})
+
+
 def format_measure_timestamp(measurements):
     # returns newline separated times for the timestamp column
     return '\n'.join([str(m[0]) for m in measurements])
@@ -410,8 +495,8 @@ def do_metric_statistics(mc, args):
     statlist = args.statistics.split(',')
     for stat in statlist:
         if stat.upper() not in statistic_types:
-            errmsg = 'Invalid type, not one of [' + \
-                ', '.join(statistic_types) + ']'
+            errmsg = ('Invalid type, not one of [' +
+                      ', '.join(statistic_types) + ']')
             print(errmsg)
             return
     fields = {}
@@ -503,11 +588,7 @@ def _validate_notification_period(period, notification_type):
            help='A period for the notification method. Can only be non zero with webhooks')
 def do_notification_create(mc, args):
     '''Create notification.'''
-    if args.type.upper() not in notification_types:
-        errmsg = 'Invalid type, not one of [' + \
-            ', '.join(notification_types) + ']'
-        print(errmsg)
-        return
+
     fields = {}
     fields['name'] = args.name
     fields['type'] = args.type
@@ -527,7 +608,7 @@ def do_notification_create(mc, args):
 
 
 @utils.arg('id', metavar='<NOTIFICATION_ID>',
-           help='The ID of the notification. If not specified returns all.')
+           help='The ID of the notification.')
 def do_notification_show(mc, args):
     '''Describe the notification.'''
     fields = {}
@@ -644,11 +725,7 @@ def do_notification_update(mc, args):
     fields = {}
     fields['notification_id'] = args.id
     fields['name'] = args.name
-    if args.type.upper() not in notification_types:
-        errmsg = 'Invalid type, not one of [' + \
-                 ', '.join(state_types) + ']'
-        print(errmsg)
-        return
+
     fields['type'] = args.type
     fields['address'] = args.address
     if not _validate_notification_period(args.period, args.type.upper()):
@@ -664,10 +741,45 @@ def do_notification_update(mc, args):
         print(jsonutils.dumps(notification, indent=2))
 
 
+@utils.arg('id', metavar='<NOTIFICATION_ID>',
+           help='The ID of the notification.')
+@utils.arg('--name', metavar='<NOTIFICATION_NAME>',
+           help='Name of the notification.')
+@utils.arg('--type', metavar='<TYPE>',
+           help='The notification type. Type must be either EMAIL, WEBHOOK, or PAGERDUTY.')
+@utils.arg('--address', metavar='<ADDRESS>',
+           help='A valid EMAIL Address, URL, or SERVICE KEY.')
+@utils.arg('--period', metavar='<PERIOD>', type=int,
+           help='A period for the notification method. Can only be non zero with webhooks')
+def do_notification_patch(mc, args):
+    '''Patch notification.'''
+    fields = {}
+    fields['notification_id'] = args.id
+    if args.name:
+        fields['name'] = args.name
+
+    fields['type'] = args.type
+    if args.address:
+        fields['address'] = args.address
+    if args.period or args.period == 0:
+        if args.type and not _validate_notification_period(
+                args.period, args.type.upper()):
+            return
+        fields['period'] = args.period
+    try:
+        notification = mc.notifications.patch(**fields)
+    except exc.HTTPException as he:
+        raise exc.CommandError(
+            'HTTPException code=%s message=%s' %
+            (he.code, he.message))
+    else:
+        print(jsonutils.dumps(notification, indent=2))
+
+
 def _validate_severity(severity):
     if severity.upper() not in severity_types:
-        errmsg = 'Invalid severity, not one of [' + \
-            ', '.join(severity_types) + ']'
+        errmsg = ('Invalid severity, not one of [' +
+                  ', '.join(severity_types) + ']')
         print(errmsg)
         return False
     return True
@@ -891,8 +1003,8 @@ def do_alarm_definition_update(mc, args):
     fields['ok_actions'] = _arg_split_patch_update(args.ok_actions)
     fields['undetermined_actions'] = _arg_split_patch_update(args.undetermined_actions)
     if args.actions_enabled not in enabled_types:
-        errmsg = 'Invalid value, not one of [' + \
-            ', '.join(enabled_types) + ']'
+        errmsg = ('Invalid value, not one of [' +
+                  ', '.join(enabled_types) + ']')
         print(errmsg)
         return
     fields['actions_enabled'] = args.actions_enabled in ['true', 'True']
@@ -952,8 +1064,8 @@ def do_alarm_definition_patch(mc, args):
         fields['undetermined_actions'] = _arg_split_patch_update(args.undetermined_actions, patch=True)
     if args.actions_enabled:
         if args.actions_enabled not in enabled_types:
-            errmsg = 'Invalid value, not one of [' + \
-                ', '.join(enabled_types) + ']'
+            errmsg = ('Invalid value, not one of [' +
+                      ', '.join(enabled_types) + ']')
             print(errmsg)
             return
         fields['actions_enabled'] = args.actions_enabled in ['true', 'True']
@@ -975,12 +1087,13 @@ def do_alarm_definition_patch(mc, args):
            help='The ID of the alarm definition.')
 @utils.arg('--metric-name', metavar='<METRIC_NAME>',
            help='Name of the metric.')
-@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
-           help='key value pair used to specify a metric dimension. '
-           'This can be specified multiple times, or once with parameters '
-           'separated by a comma. '
-           'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
-           'that confuse the CLI parser.',
+@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2,KEY3=VALUE2...>',
+           help='key value pair used to specify a metric dimension or '
+                'just key to select all values of that dimension.'
+                'This can be specified multiple times, or once with parameters '
+                'separated by a comma. '
+                'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
+                'that confuse the CLI parser.',
            action='append')
 @utils.arg('--state', metavar='<ALARM_STATE>',
            help='ALARM_STATE is one of [UNDETERMINED, OK, ALARM].')
@@ -1013,8 +1126,8 @@ def do_alarm_list(mc, args):
         fields['metric_dimensions'] = utils.format_dimensions_query(args.metric_dimensions)
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1121,8 +1234,8 @@ def do_alarm_update(mc, args):
     fields = {}
     fields['alarm_id'] = args.id
     if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
     fields['state'] = args.state
@@ -1152,8 +1265,8 @@ def do_alarm_patch(mc, args):
     fields['alarm_id'] = args.id
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1218,17 +1331,20 @@ def output_alarm_history(args, alarm_history):
            help='The ID of the alarm definition.')
 @utils.arg('--metric-name', metavar='<METRIC_NAME>',
            help='Name of the metric.')
-@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
-           help='key value pair used to specify a metric dimension. '
-           'This can be specified multiple times, or once with parameters '
-           'separated by a comma. '
-           'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
-           'that confuse the CLI parser.',
+@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2,KEY3=VALUE2...>',
+           help='key value pair used to specify a metric dimension or '
+                'just key to select all values of that dimension.'
+                'This can be specified multiple times, or once with parameters '
+                'separated by a comma. '
+                'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
+                'that confuse the CLI parser.',
            action='append')
 @utils.arg('--state', metavar='<ALARM_STATE>',
            help='ALARM_STATE is one of [UNDETERMINED, OK, ALARM].')
 @utils.arg('--severity', metavar='<SEVERITY>',
            help='Severity is one of ["LOW", "MEDIUM", "HIGH", "CRITICAL"].')
+@utils.arg('--state-updated-start-time', metavar='<UTC_STATE_UPDATED_START>',
+           help='Return all alarms whose state was updated on or after the time specified.')
 @utils.arg('--lifecycle-state', metavar='<LIFECYCLE_STATE>',
            help='The lifecycle state of the alarm.')
 @utils.arg('--link', metavar='<LINK>',
@@ -1249,11 +1365,11 @@ def do_alarm_count(mc, args):
     if args.metric_name:
         fields['metric_name'] = args.metric_name
     if args.metric_dimensions:
-        fields['metric_dimensions'] = utils.format_parameters(args.metric_dimensions)
+        fields['metric_dimensions'] = utils.format_dimensions_query(args.metric_dimensions)
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1261,6 +1377,8 @@ def do_alarm_count(mc, args):
         if not _validate_severity(args.severity):
             return
         fields['severity'] = args.severity
+    if args.state_updated_start_time:
+        fields['state_updated_start_time'] = args.state_updated_start_time
     if args.lifecycle_state:
         fields['lifecycle_state'] = args.lifecycle_state
     if args.link:
@@ -1268,8 +1386,8 @@ def do_alarm_count(mc, args):
     if args.group_by:
         group_by = args.group_by.split(',')
         if not set(group_by).issubset(set(group_by_types)):
-            errmsg = 'Invalid group-by, one or more values not in [' + \
-                ','.join(group_by_types) + ']'
+            errmsg = ('Invalid group-by, one or more values not in [' +
+                      ','.join(group_by_types) + ']')
             print(errmsg)
             return
         fields['group_by'] = args.group_by
@@ -1356,6 +1474,25 @@ def do_alarm_history_list(mc, args):
         output_alarm_history(args, alarm)
 
 
+def do_notification_type_list(mc, args):
+    '''List notification types supported by monasca.'''
+
+    try:
+        notification_types = mc.notificationtypes.list()
+    except exc.HTTPException as he:
+        raise exc.CommandError(
+            'HTTPException code=%s message=%s' %
+            (he.code, he.message))
+    else:
+        if args.json:
+            print(utils.json_formatter(notification_types))
+            return
+        else:
+            formatters = {'types': lambda x: x["type"]}
+            # utils.print_list(notification_types['types'], ["types"], formatters=formatters)
+            utils.print_list(notification_types, ["types"], formatters=formatters)
+
+
 def _translate_starttime(args):
     if args.starttime[0] == '-':
         deltaT = time.time() + (int(args.starttime) * 60)
